/*
** OpenAmiga.org
**  This file is part of the OpenAmiga "XAD filesystem"
**
** For more information look at:
**   http://openamiga.org/?function=viewproject&projectid=66
*/

#include "xadfs.h"

int32 FSChangeFilePosition(struct FSVP *vp, int32 *res2, struct FileHandle *fh,
	int32 mode, int64 pos)
{
	struct GlobalData *gd = vp->FSV.FSPrivate;
	struct ObjLock *lock;
	struct ObjNode *obj;
	int64 new_pos = -1;
	int32 result = DOSFALSE;

	if (fh) {
		IEXEC->ObtainSemaphore(gd->Sem);

		lock = fh->fh_Arg2;
		obj = lock->node;

		if (NO lock->cache_file && obj->file_size == -1) {
			lock->cache_file = get_cache_file(gd, res2, lock->node);
			if (lock->cache_file && lock->current_position) {
				if (NOT IDOS->ChangeFilePosition(lock->cache_file,
					lock->current_position, OFFSET_BEGINNING))
				{
					IDOS->Close(lock->cache_file);
					lock->cache_file = ZERO;
					*res2 = ERROR_SEEK_ERROR;
				}
			}
		}

		if (lock->cache_file) {
			result = IDOS->ChangeFilePosition(lock->cache_file, pos, mode);
			if (NOT result) {
				*res2 = IDOS->IoErr();
			}
		} else if (obj->file_size != -1) {
			switch (mode) {
			case OFFSET_BEGINNING:
				new_pos = pos;
				break;
			case OFFSET_CURRENT:
				new_pos = lock->current_position + pos;
				break;
			case OFFSET_END:
				new_pos = obj->file_size + pos;
				break;
			}
			if (new_pos >= 0 && new_pos <= obj->file_size) {
				lock->current_position = new_pos;
				result = DOSTRUE;
			} else {
				*res2 = ERROR_SEEK_ERROR;
			}
		}

		IEXEC->ReleaseSemaphore(gd->Sem);
	} else {
		*res2 = ERROR_REQUIRED_ARG_MISSING;
	}

	return result;
}

