/*
** OpenAmiga.org
**  This file is part of the OpenAmiga "XAD filesystem"
**
** For more information look at:
**   http://openamiga.org/?function=viewproject&projectid=66
*/

#include "xadfs.h"

int32 FSChangeLockMode(struct FSVP *vp, int32 *res2, struct Lock *lockin, int32 newmode) {
	struct GlobalData *gd = vp->FSV.FSPrivate;
	struct ObjLock *lock = (struct ObjLock *)lockin;
	struct ObjNode *obj;
	int32 result = DOSFALSE;

	IEXEC->ObtainSemaphore(gd->Sem);

	if (NO lock) {
		*res2 = ERROR_REQUIRED_ARG_MISSING;
		goto error;
	}

	obj = lock->node;

	if (newmode == EXCLUSIVE_LOCK) {
		if (obj->type == FSO_TYPE_DIRECTORY) {
			*res2 = ERROR_OBJECT_WRONG_TYPE;
			goto error;
		}

		if (lock->lock.fl_Access == SHARED_LOCK) {
			if (obj->lock_count != 1) {
				*res2 = ERROR_OBJECT_IN_USE;
				goto error;
			}
			obj->lock_count = -1;
			lock->lock.fl_Access = EXCLUSIVE_LOCK;
		}
	} else {
		if (lock->lock.fl_Access == EXCLUSIVE_LOCK) {
			obj->lock_count = 1;
			lock->lock.fl_Access = SHARED_LOCK;
		}
	}

	result = DOSTRUE;

error:
	IEXEC->ReleaseSemaphore(gd->Sem);

	return result;
}

