/*
** OpenAmiga.org
**  This file is part of the OpenAmiga "XAD filesystem"
**
** For more information look at:
**   http://openamiga.org/?function=viewproject&projectid=66
*/

#include "xadfs.h"

int32 FSExamineDir(struct FSVP *vp, int32 *res2, struct PRIVATE_ExamineDirContext *ctx) {
	struct GlobalData *gd = vp->FSV.FSPrivate;
	struct ExamineData *exd;
	struct ObjLock *lock;
	struct ObjNode *dirnode;
	struct ObjNode *entry;
	uint32 mask;

	IEXEC->ObtainSemaphoreShared(gd->Sem);

	if (NO ctx) {
		*res2 = ERROR_REQUIRED_ARG_MISSING;
		goto error;
	}

	lock = (struct ObjLock *)ctx->ReferenceLock;
	mask = ctx->DataRequestFlags;

	while ((exd = (struct ExamineData *)IEXEC->RemHead((struct List *)&ctx->StaleNodeList))) {
		IDOS->FreeDosObject(DOS_EXAMINEDATA, exd);
	}

	if (NO lock) {
		*res2 = ERROR_REQUIRED_ARG_MISSING;
		goto error;
	}

	dirnode = lock->node;
	if (NO dirnode) {
		*res2 = ERROR_INVALID_LOCK;
		goto error;
	}

	if (dirnode->type != FSO_TYPE_DIRECTORY) {
		*res2 = ERROR_OBJECT_WRONG_TYPE;
		goto error;
	}

	entry = (struct ObjNode *)IEXEC->AVL_FindFirstNode(dirnode->content_list);
	while (entry) {
		exd = IDOS->AllocDosObjectTags(DOS_EXAMINEDATA,
			ADO_ExamineData_LinkSize,    1,
			ADO_ExamineData_NameSize,    SAFE_STRLEN(entry->name) + 1,
			ADO_ExamineData_CommentSize, SAFE_STRLEN(entry->comment) + 1,
			ADO_ExamineDir_Context,      ctx,
			TAG_END);
		if (NO exd) {
			*res2 = ERROR_NO_FREE_STORE;
			goto error;
		}

		if (mask & EXF_TYPE) {
			exd->Type = entry->type;
		}

		if (mask & EXF_SIZE) {
			exd->FileSize = entry->file_size;
		}

		if (mask & EXF_DATE) {
			exd->Date = entry->date;
		}

		if (mask & EXF_USECOUNT) {
			exd->UseCount = 0;
		}

		if (mask & EXF_OBJECTID) {
			exd->ObjectID = MKBADDR(entry);
		}

		if (mask & EXF_NAME) {
			if (entry->name) {
				IUTILITY->Strlcpy(exd->Name, entry->name, exd->NameSize);
			} else {
				exd->Name[0] = '\0';
			}
		}

		if (mask & EXF_COMMENT) {
			if (entry->comment) {
				IUTILITY->Strlcpy(exd->Comment, entry->comment, exd->CommentSize);
			} else {
				exd->Comment[0] = '\0';
			}
		}

		if (mask & EXF_LINK) {
			exd->Link[0] = '\0';
		}

		if (mask & EXF_PROTECTION) {
			exd->Protection = entry->protection;
		}

		if (mask & EXF_OWNER) {
			exd->OwnerUID = entry->owner;
			exd->OwnerGID = entry->group;
		}

		IEXEC->AddTail((struct List *)&ctx->FreshNodeList, (struct Node *)&exd->EXDnode);

		entry = (struct ObjNode *)IEXEC->AVL_FindNextNodeByAddress(&entry->node);
	}

	*res2 = ERROR_NO_MORE_ENTRIES;

error:
	IEXEC->ReleaseSemaphore(gd->Sem);

	return DOSFALSE;
}

