/*
** OpenAmiga.org
**  This file is part of the OpenAmiga "XAD filesystem"
**
** For more information look at:
**   http://openamiga.org/?function=viewproject&projectid=66
*/

#include "xadfs.h"

struct ExamineData *FSExamineLock(struct FSVP *vp, int32 *res2, struct Lock *lockin) {
	struct GlobalData *gd = vp->FSV.FSPrivate;
	struct ObjLock *lock = (struct ObjLock *)lockin;
	struct ObjNode *obj;
	struct ExamineData *exd = NULL;

	IEXEC->ObtainSemaphoreShared(gd->Sem);

	if (NO lock) {
		*res2 = ERROR_REQUIRED_ARG_MISSING;
		goto error;
	}

	obj = lock->node;
	if (NO obj) {
		*res2 = ERROR_INVALID_LOCK;
		goto error;
	}

	exd = IDOS->AllocDosObjectTags(DOS_EXAMINEDATA,
		ADO_ExamineData_NameSize,    SAFE_STRLEN(obj->name) + 1,
		ADO_ExamineData_CommentSize, SAFE_STRLEN(obj->comment) + 1,
		TAG_END);
	if (NO exd) {
		*res2 = ERROR_NO_FREE_STORE;
		goto error;
	}

	exd->Type = obj->type;
	exd->FileSize = obj->file_size;

	if (obj->name) {
		IUTILITY->Strlcpy(exd->Name, obj->name, exd->NameSize);
	} else {
		exd->Name[0] = '\0';
	}

	if (obj->comment) {
		IUTILITY->Strlcpy(exd->Comment, obj->comment, exd->CommentSize);
	} else {
		exd->Comment[0] = '\0';
	}

	exd->Date = obj->date;

	exd->UseCount   = 0;
	exd->ObjectID   = MKBADDR(obj);
	exd->Protection = obj->protection;
	exd->OwnerUID   = obj->owner;
	exd->OwnerGID   = obj->group;

error:
	IEXEC->ReleaseSemaphore(gd->Sem);

	return exd;
}

