/*
** OpenAmiga.org
**  This file is part of the OpenAmiga "XAD filesystem"
**
** For more information look at:
**   http://openamiga.org/?function=viewproject&projectid=66
*/

#include "xadfs.h"
#include "XADFileSystem_rev.h"

int32 FSFileSystemAttr(struct FSVP *vp, int32 *res2, struct TagItem *tags) {
	struct GlobalData *gd = vp->FSV.FSPrivate;
	struct TagItem *tstate, *tag;
	uint32 *data;
	STRPTR cp;
	int32 bufsize, len;
	int32 rc1 = DOSTRUE;
	int32 rc2 = 0;

	IEXEC->ObtainSemaphoreShared(gd->Sem);

	tstate = tags;
	while ((tag = IUTILITY->NextTagItem(&tstate))) {
		data = (uint32 *)tag->ti_Data;

		switch (tag->ti_Tag) {
		case FSA_MaxFileNameLengthR:
			*data = MAX_FILENAME;
			break;

		case FSA_VersionNumberR:
			*data = (VERSION << 16) | REVISION;
			break;

		case FSA_DOSTypeR:
			*data = gd->volume_node->vn_DiskType;
			break;

		case FSA_VersionStringR:
			bufsize = IUTILITY->GetTagData(FSA_VersionStringR_BufSize, 0, tags); 
			len = sizeof(VSTRING);

			if (len > bufsize) {
				rc2 = ERROR_LINE_TOO_LONG;
				rc1 = DOSFALSE;
				break;
			}
			IUTILITY->Strlcpy((STRPTR)data, VSTRING, bufsize);
			/* trim off newlines, etc... */
			for (cp = (STRPTR)data; *cp; cp++) {
				if ((*cp == '\n') || (*cp == '\r')) {
					*cp = '\0';
					break;
				}
			}
			break;

		/* these return success with 0 value */
		case FSA_HasRecycledEntriesR:
		case FSA_ActivityFlushTimeoutR:
		case FSA_InactivityFlushTimeoutR:
		case FSA_MaxRecycledEntriesR:
			*data = 0;
			break;

		/* these are ignored */
		case FSA_VersionStringR_BufSize: /* gotten previously */
		case FSA_StringNameInput:
   		case FSA_FileHandleInput:
   		case FSA_LockInput:
   		case FSA_MsgPortInput:
			break;

		/* these are unimplemented */
		case FSA_MaxRecycledEntriesW:
		case FSA_InactivityFlushTimeoutW:
		case FSA_ActivityFlushTimeoutW:
		case FSA_MaxFileNameLengthW:
		default:
			rc2 = ERROR_NOT_IMPLEMENTED;
			rc1 = DOSFALSE;
			break;
		}

		if (rc1 == DOSFALSE) {
			break;
		}
	}

	IEXEC->ReleaseSemaphore(gd->Sem);

	*res2 = rc2;
	return rc1;
}

