/*
** OpenAmiga.org
**  This file is part of the OpenAmiga "XAD filesystem"
**
** For more information look at:
**   http://openamiga.org/?function=viewproject&projectid=66
*/

#include "xadfs.h"

int64 FSGetFileSize(struct FSVP *vp, int32 *res2, struct FileHandle *fh) {
	struct GlobalData *gd = vp->FSV.FSPrivate;
	struct ObjLock *lock;
	struct ObjNode *obj;
	int64 result = -1;

	if (fh) {
		IEXEC->ObtainSemaphore(gd->Sem);

		lock = fh->fh_Arg2;
		obj = lock->node;

		if (NO lock->cache_file && obj->file_size == -1) {
			lock->cache_file = get_cache_file(gd, res2, lock->node);
			if (lock->cache_file && lock->current_position) {
				if (NOT IDOS->ChangeFilePosition(lock->cache_file,
					lock->current_position, OFFSET_BEGINNING))
				{
					IDOS->Close(lock->cache_file);
					lock->cache_file = ZERO;
					*res2 = ERROR_SEEK_ERROR;
				}
			}
		}

		if (lock->cache_file) {
			result = IDOS->GetFileSize(lock->cache_file);
			if (result == -1) {
				*res2 = IDOS->IoErr();
			}
		} else if (obj->file_size != -1) {
			result = obj->file_size;
		}

		IEXEC->ReleaseSemaphore(gd->Sem);
	} else {
		*res2 = ERROR_REQUIRED_ARG_MISSING;
		result = -1;
	}

	return result;
}

