/*
** OpenAmiga.org
**  This file is part of the OpenAmiga "XAD filesystem"
**
** For more information look at:
**   http://openamiga.org/?function=viewproject&projectid=66
*/

#include "xadfs.h"

struct ObjLock *new_lock(struct GlobalData *gd, int32 *res2, struct ObjNode *obj, int32 mode) {
	struct ObjLock *lock;

	if (obj->type == FSO_TYPE_DIRECTORY && mode == EXCLUSIVE_LOCK) {
		mode = SHARED_LOCK;
	}

	if (mode == EXCLUSIVE_LOCK) {
		if (obj->lock_count != 0) {
			*res2 = ERROR_OBJECT_IN_USE;
			return NULL;
		}
	} else {
		if (obj->lock_count < 0) {
			*res2 = ERROR_OBJECT_IN_USE;
			return NULL;
		}
	}

	lock = IDOS->AllocDosObjectTags(DOS_LOCK,
		ADO_DOSType, gd->volume_node->vn_DiskType,
		ADO_Size,    sizeof(*lock),
		TAG_END);
	if (NO lock) {
		*res2 = ERROR_NO_FREE_STORE;
		return NULL;
	}

	if (mode == EXCLUSIVE_LOCK) {
		obj->lock_count = -1;
		lock->lock.fl_Access = EXCLUSIVE_LOCK;
	} else {
		obj->lock_count++;
		lock->lock.fl_Access = SHARED_LOCK;
	}

	lock->lock.fl_Link     = gd->lock_list;
	lock->lock.fl_Key      = MKBADDR(obj);
	lock->lock.fl_Port     = &gd->FSVectorPort->MP;
	lock->lock.fl_Volume   = MKBADDR(gd->volume_node);
	lock->node             = obj;
	lock->fh               = NULL;
	lock->cache_file       = ZERO;
	lock->current_position = 0;

	gd->lock_list = MKBADDR(lock);

	return lock;
}

struct Lock *FSLock(struct FSVP *vp, int32 *res2, struct Lock *rel_lock,
	CONST_STRPTR name, int32 mode)
{
	struct GlobalData *gd = vp->FSV.FSPrivate;
	struct ObjNode *obj;
	struct Lock *result = NULL;

	IEXEC->ObtainSemaphore(gd->Sem);

	obj = handle_rel_args(gd, res2, rel_lock, name);
	if (obj) {
		result = (struct Lock *)new_lock(gd, res2, obj, mode);
	}

	IEXEC->ReleaseSemaphore(gd->Sem);

	return result;
}

