/*
** OpenAmiga.org
**  This file is part of the OpenAmiga "XAD filesystem"
**
** For more information look at:
**   http://openamiga.org/?function=viewproject&projectid=66
*/

#include "xadfs.h"

int32 FSRead(struct FSVP *vp, int32 *res2, struct FileHandle *fh,
	STRPTR buffer, int32 numbytes)
{
	struct GlobalData *gd = vp->FSV.FSPrivate;
	struct ObjLock *lock;
	int32 result = -1;

	if (fh) {
		IEXEC->ObtainSemaphore(gd->Sem);

		lock = fh->fh_Arg2;

		if (NO lock->cache_file) {
			lock->cache_file = get_cache_file(gd, res2, lock->node);
			if (lock->cache_file && lock->current_position) {
				if (NOT IDOS->ChangeFilePosition(lock->cache_file,
					lock->current_position, OFFSET_BEGINNING))
				{
					IDOS->Close(lock->cache_file);
					lock->cache_file = ZERO;

					*res2 = ERROR_SEEK_ERROR;
				}
			}
		}

		if (lock->cache_file) {
			result = IDOS->Read(lock->cache_file, buffer, numbytes);
			if (result != -1) {
				*res2 = 0;
			} else {
				*res2 = IDOS->IoErr();
			}
		}

		IEXEC->ReleaseSemaphore(gd->Sem);
	} else {
		*res2 = ERROR_REQUIRED_ARG_MISSING;
	}

	return result;
}

