/*
** OpenAmiga.org
**  This file is part of the OpenAmiga "XAD filesystem"
**
** For more information look at:
**   http://openamiga.org/?function=viewproject&projectid=66
*/

#include "xadfs.h"

static int32 free_lock(struct GlobalData *gd, int32 *res2, struct ObjLock *lock) {
	struct ObjLock *last_lock, *this_lock;
	struct ObjNode *obj;

	last_lock = NULL;
	this_lock = BADDR(gd->lock_list);
	while (this_lock) {
		if (this_lock == lock) {
			if (last_lock) {
				last_lock->lock.fl_Link = lock->lock.fl_Link;
			} else {
				gd->lock_list = lock->lock.fl_Link;
			}
			break;
		}
		last_lock = this_lock;
		this_lock = BADDR(this_lock->lock.fl_Link);
	}
	if (NO this_lock) {
		*res2 = ERROR_INVALID_LOCK; /* couldn't find lock in list */
		return DOSFALSE;
	}

	obj = lock->node;
	if (lock->lock.fl_Access == EXCLUSIVE_LOCK) {
		obj->lock_count = 0;
	} else {
		obj->lock_count--;
	}

	IDOS->Close(lock->cache_file);

	lock->node             = NULL;
	lock->fh               = NULL;
	lock->cache_file       = ZERO;
	lock->current_position = -1;

	IDOS->FreeDosObject(DOS_LOCK, lock);

	return DOSTRUE;
}

int32 FSUnLock(struct FSVP *vp, int32 *res2, struct Lock *lock) {
	struct GlobalData *gd = vp->FSV.FSPrivate;
	int32 result;

	IEXEC->ObtainSemaphore(gd->Sem);

	result = free_lock(gd, res2, (struct ObjLock *)lock);

	IEXEC->ReleaseSemaphore(gd->Sem);

	return result;
}

