/*
** OpenAmiga.org
**  This file is part of the OpenAmiga "XAD filesystem"
**
** For more information look at:
**   http://openamiga.org/?function=viewproject&projectid=66
*/

#include <workbench/startup.h>
#include <proto/dos.h>
#include <proto/utility.h>
#include <string.h>
#include "MountXAD_rev.h"

#define PROGNAME "MountXAD"
CONST TEXT USED verstag[] = VERSTAG;

#define TEMPLATE "FILE/A,DISMOUNT/S"

struct WBStartup *WBenchMsg;

struct VolumeNode *FindVolumeNode (CONST_STRPTR name, struct DateStamp *date, STRPTR devname);

int main (int argc, char **argv) {
	int rc = RETURN_FAIL;
	CONST_STRPTR filename;
	BOOL dismount = FALSE;
	struct RDArgs *rda = NULL;
	struct ExamineData *ed = NULL;
	BPTR dir = ZERO;
	
	if (argc == 0) {
		WBenchMsg = (struct WBStartup *)argv;
		if (WBenchMsg->sm_NumArgs != 2) {
			return rc;
		}
		dir = IDOS->CurrentDir(WBenchMsg->sm_ArgList[1].wa_Lock);
		filename = WBenchMsg->sm_ArgList[1].wa_Name;
	} else {
		int32 array[2];
		IUtility->ClearMem(array, sizeof(array));
		rda = IDOS->ReadArgs(TEMPLATE, array, NULL);
		if (!rda) {
			IDOS->PrintFault(IDOS->IoErr(), PROGNAME);
			goto error;
		}
		filename = (CONST_STRPTR)array[0];
		dismount = array[1] ? TRUE : FALSE;
	}

	ed = IDOS->ExamineObjectTags(EX_StringNameInput, filename, TAG_END);
	if (!ed) {
		IDOS->PrintFault(IDOS->IoErr(), filename);
		goto error;
	}
	if (argc == 0) {
		dismount = FindVolumeNode(ed->Name, &ed->Date, NULL) ? TRUE : FALSE;
	}

	if (!dismount) {
		BPTR file;
		TEXT fullpath[512];
		const uint32 flags = LDF_DEVICES|LDF_READ;
		struct DosList *dl;
		int32 n = 0;
		TEXT devname[16];
		APTR window;
		int32 result;

		if (FindVolumeNode(ed->Name, &ed->Date, NULL)) {
			IDOS->Printf("%s: Archive already mounted\n", ed->Name);
			goto error;
		}

		file = IDOS->Open(filename, MODE_OLDFILE);
		if (!file) {
			IDOS->PrintFault(IDOS->IoErr(), filename);
			goto error;
		}
		if (!IDOS->DevNameFromFH(file, fullpath, sizeof(fullpath), DN_FULLPATH)) {
			IDOS->PrintFault(IDOS->IoErr(), PROGNAME);
			IDOS->Close(file);
			goto error;
		}
		IDOS->Close(file);

		do {
			IUtility->SNPrintf(devname, sizeof(devname), "XAD%ld", n++);
			dl = IDOS->LockDosList(flags);
			dl = IDOS->FindDosEntry(dl, devname, flags);
			IDOS->UnLockDosList(flags);
		} while (dl);
		IUtility->Strlcat(devname, ":", sizeof(devname));

		window = IDOS->SetProcWindow((APTR)-1);
		result = IDOS->MountDeviceTags(devname, MDT_Handler,
			MD_Handler,			"L:XADFileSystem",
			MD_StartupString,	fullpath,
			MD_Activate,		TRUE,
			TAG_END);
		IDOS->SetProcWindow(window);
		if (!result) {
			int32 error = IDOS->IoErr();
			if (error == ERROR_DEVICE_NOT_MOUNTED) {
				IDOS->DismountDevice(devname, DMDF_REMOVEDEVICE, NULL);
			}
			IDOS->PrintFault(error, PROGNAME);
			goto error;
		}
	} else {
		struct VolumeNode *volume;
		TEXT devname[16];
		int32 result;

		volume = FindVolumeNode(ed->Name, &ed->Date, devname);
		if (!volume) {
			IDOS->PrintFault(ERROR_DEVICE_NOT_MOUNTED, ed->Name);
			goto error;
		}
		IUtility->Strlcat(devname, ":", sizeof(devname));

		result = IDOS->DismountDevice(devname, DMDF_REMOVEDEVICE, NULL);
		if (!result) {
			IDOS->PrintFault(IDOS->IoErr(), PROGNAME);
			goto error;
		}
	}
	rc = RETURN_OK;

error:
	IDOS->FreeDosObject(DOS_EXAMINEDATA, ed);
	if (argc == 0) {
		IDOS->CurrentDir(dir);
	} else {
		IDOS->FreeArgs(rda);
	}
	return rc;
}

struct VolumeNode *FindVolumeNode (CONST_STRPTR name, struct DateStamp *date, STRPTR devname) {
	const uint32 flags = LDF_READ|LDF_VOLUMES;
	struct DosList *dl;
	struct VolumeNode *volume = NULL;
	if (devname) {
		devname[0] = 0;
	}
	dl = IDOS->LockDosList(flags);
	while ((dl = IDOS->FindDosEntry(dl, name, flags))) {
		if (!memcmp(date, &dl->dol_misc.dol_volume.dol_VolumeDate, sizeof(struct DateStamp))) {
			volume = (struct VolumeNode *)dl;
			if (devname) {
				IDOS->CopyStringBSTRToC(volume->vn_FSPrivate, devname, 16);
			}
			break;
		}
		dl = IDOS->NextDosEntry(dl, flags);
	}
	IDOS->UnLockDosList(flags);
	return volume;
}
