/*
** OpenAmiga.org
**  This file is part of the OpenAmiga "XAD filesystem"
**
** For more information look at:
**   http://openamiga.org/?function=viewproject&projectid=66
*/

#ifndef XADFS_H
#define XADFS_H

#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/utility.h>
#include <proto/xadmaster.h>

#define ID_XAD0_DISK (0x58414400L) /* 'XAD\0' */

#ifndef NOT
#define NOT !
#endif

#ifndef NO
#define NO  !
#endif

#ifndef SAME
#define SAME 0
#endif

#define IEXEC    (gd->IExec)
#define IDOS     (gd->IDos)
#define IUTILITY (gd->IUtility)
#define IXAD     (gd->IXad)

#define MAX_FILENAME MAX_VP_FILENAME
#define MAX_COMMENT  MAX_VP_COMMENT

#define SAFE_STRLEN(str) ((str) ? (IUTILITY->Strlen(str)) : 0)

struct ObjNode {
	struct AVLNode node;
	struct GlobalData *global_data; /* needed for AVL callbacks */

	uint32       type;
	uint32       name_size;
	CONST_STRPTR name;

	struct AVLNode  *content_list;
	struct ObjNode  *parent_dir;
	int64            file_size; /* -1 if directory or unknown size */
	uint32           protection;
	uint32           owner; /* currently 0 */
	uint32           group; /* currently 0 */
	struct DateStamp date;
	CONST_STRPTR     comment;

	struct xadFileInfo *xad_file_info;

	int32 lock_count; /* -1 for exclusive lock */
};

struct ObjLock {
	struct Lock lock;

	struct ObjNode *node;

	struct FileHandle *fh;

	BPTR cache_file;

	int64 current_position;
};

struct GlobalData {
	struct Library *SysBase;
	struct Library *DOSBase;
	struct Library *UtilityBase;
	struct Library *XADBase;

	struct ExecIFace      *IExec;
	struct DOSIFace       *IDos;
	struct UtilityIFace   *IUtility;
	struct XadMasterIFace *IXad;

	int32 debug_level;

	struct SignalSemaphore *Sem;

	BPTR lock_list;

	struct ObjNode *root_node;

	struct VolumeNode *volume_node;
	struct DeviceNode *device_node;

	struct FileSystemVectorPort *FSVectorPort;

	APTR memory_pool;

	BPTR archive_file;
	struct xadArchiveInfo *xad_archive_info;

	int64 disk_size;

	CONST_STRPTR device_name;
	BPTR         cache_dir_lock;
};

/* xad.c */
int32 mount_xad_archive(struct GlobalData *gd, CONST_STRPTR archiveName);
void unmount_xad_archive(struct GlobalData *gd);
struct ObjNode *handle_rel_args(struct GlobalData *gd, int32 *res2,
	struct Lock *rel_lock, CONST_STRPTR name);
BPTR get_cache_file(struct GlobalData *gd, int32 *res2, struct ObjNode *obj);

/* _FSLock.c */
struct ObjLock *new_lock(struct GlobalData *gd, int32 *res2, struct ObjNode *obj, int32 mode);

#endif

